# Adapted from John Ousterhout's example (mainly to use better arg passing :)

# Now has a global variable for each dialog box (thus .query has
# button_result_.query as it's variable :), and uses X resources to
# configure the things that might reasonably be set by them (the font
# and the width of the actual written message).
#
# I'll fill in the args, bugs, etc. sometime when I'm bored...

proc dialog {w args} {
    upvar #0 button_result_$w mybut
    
    # 0. Parse the command line for arguements

    set want(-title)   1
    set want(-text)    1
    set want(-bitmap)  1
    set want(-default) 2; set got(-default) -1
    set args [parseargs want got $args]

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    toplevel $w -class DKFDialog
    
    if [regexp {^\.[^.]+$} $w] {
	set parent .
    } else {
	regexp {^(.+)\.[^.]+$} $w wibble parent
	set parent [winfo toplevel $parent]
    }

    catch {
	wm transient $w $parent
	wm group     $w $parent
    }
    catch {
	regexp {([-+][0-9]+)([-+][0-9]+)} [wm geometry $parent] g geomX geomY
	if {$geomX < 0} {
	    set geomX +0
	}
	if {$geomY < 0} {
	    set geomY +0
	}
	wm geometry $w +[expr 30 $geomX]+[expr 30 $geomY]
    }
    catch {
	wm title $w $got(-title)
	wm iconname $w Dialog
    }
    frame $w.top -relief raised -bd 1
    frame $w.bot -relief raised -bd 1
    pack $w.top -side top    -fill both
    pack $w.bot -side bottom -fill both

    # 2. Fill the top part with the bitmap and message.

    message $w.top.msg -text $got(-text)
    pack $w.top.msg -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$got(-bitmap) != ""} {
	label $w.top.bitmap -bitmap $got(-bitmap)
	pack $w.top.bitmap -side left -padx 3m -pady 3m
    }

    # 3. Create a row of buttons at the bottom of the dialog.
    
    set i 0
    foreach but $args {
	button $w.bot.button$i -text $but -command "dialog_button_response $w $i"
	if {$i == $got(-default)} {
	    frame $w.bot.default -relief sunken -bd 1
	    raise $w.bot.button$i
	    pack $w.bot.default  -expand 1          -side left -padx 3m -pady 2m
	    pack $w.bot.button$i -in $w.bot.default -side left -padx 2m -pady 2m -ipadx 2m -ipady 1m
	} else {
	    pack $w.bot.button$i -expand 1          -side left -padx 3m -pady 3m -ipadx 2m -ipady 1m
	}
	incr i
    }

    # 4. Set up a binding for <Return>, if there`s a default,
    # set a grab, and claim the focus too.

    if {$got(-default) >= 0} {
	bind $w <Return> "dialog_button_response $w $got(-default) 1"
    }
    set oldFocus [focus]
    update idletasks
    grab set $w
    focus $w
    
    # 5. Wait for the user to respond, then restore the focus
    # and return the index of the selected button.

    tkwait window $w
    catch {
	focus $oldFocus
    }
    return $mybut
}

### --- PRIVATE PROCEDURES... ---------------------------------------------------

;proc dialog_button_response {win val {flash 0}} {
    upvar #0 button_result_$win mybut
    if $flash {
	$win.bot.button$val flash
    }
    set mybut $val;
    destroy $win
}

option add  "*DKFDialog.top.msg.font"   "-Adobe-Times-Medium-R-Normal-*-180-*"  startup
option add  "*DKFDialog.top.msg.width"  "3i"                                    startup
